@@ -137,8 +137,6 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
     def post(self, *args, **kwargs):
         """Create an object
         """
-        self.before_post(args, kwargs)
-
         json_data = request.get_json()
 
         qs = QSManager(request.args, self.schema)
@@ -169,6 +167,8 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
                 error['title'] = "Validation error"
             return errors, 422
 
+        self.before_post(args, kwargs, data=data)
+
         obj = self._data_layer.create_object(data, kwargs)
 
         result = schema.dump(obj).data
@@ -181,7 +181,7 @@ class ResourceList(with_metaclass(ResourceMeta, Resource)):
     def after_get(self, result):
         pass
 
-    def before_post(self, args, kwargs):
+    def before_post(self, args, kwargs, data=None):
         pass
 
     def after_post(self, result):
@@ -214,8 +214,6 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
     def patch(self, *args, **kwargs):
         """Update an object
         """
-        self.before_patch(args, kwargs)
-
         json_data = request.get_json()
 
         qs = QSManager(request.args, self.schema)
@@ -253,6 +251,8 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
         if json_data['data']['id'] != str(kwargs[self.data_layer.get('url_field', 'id')]):
             raise BadRequest('/data/id', 'Value of id does not match the resource identifier in url')
 
+        self.before_patch(args, kwargs, data=data)
+
         obj = self._data_layer.get_object(kwargs)
         self._data_layer.update_object(obj, data, kwargs)
 
@@ -270,7 +270,7 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
         obj = self._data_layer.get_object(kwargs)
         self._data_layer.delete_object(obj, kwargs)
 
-        result = {'meta': 'Object successful deleted'}
+        result = {'meta': {'message': 'Object successfully deleted'}}
         self.after_delete(result)
         return result
 
@@ -280,7 +280,7 @@ class ResourceDetail(with_metaclass(ResourceMeta, Resource)):
     def after_get(self, result):
         pass
 
-    def before_patch(self, args, kwargs):
+    def before_patch(self, args, kwargs, data=None):
         pass
 
     def after_patch(self, result):
@@ -302,23 +302,14 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
         self.before_get(args, kwargs)
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
-        related_view = self.schema._declared_fields[relationship_field].related_view
-        related_view_kwargs = self.schema._declared_fields[relationship_field].related_view_kwargs
 
         obj, data = self._data_layer.get_relationship(model_relationship_field,
                                                       related_type_,
                                                       related_id_field,
                                                       kwargs)
 
-        for key, value in copy(related_view_kwargs).items():
-            if isinstance(value, str) and value.startswith('<') and value.endswith('>'):
-                tmp_obj = obj
-                for field in value[1:-1].split('.'):
-                    tmp_obj = getattr(tmp_obj, field)
-                related_view_kwargs[key] = tmp_obj
-
         result = {'links': {'self': request.path,
-                            'related': url_for(related_view, **related_view_kwargs)},
+                            'related': self.schema._declared_fields[relationship_field].get_related_url(obj)},
                   'data': data}
 
         qs = QSManager(request.args, self.schema)
@@ -335,8 +326,6 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
     def post(self, *args, **kwargs):
         """Add / create relationship(s)
         """
-        self.before_post(args, kwargs)
-
         json_data = request.get_json()
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
@@ -359,6 +348,8 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
                 if obj['type'] != related_type_:
                     raise InvalidType('/data/type', 'The type provided does not match the resource type')
 
+        self.before_post(args, kwargs, json_data=json_data)
+
         obj_, updated = self._data_layer.create_relationship(json_data,
                                                              model_relationship_field,
                                                              related_id_field,
@@ -370,19 +361,19 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
             includes.append(relationship_field)
         schema = compute_schema(self.schema, dict(), qs, includes)
 
-        status_code = 200 if updated is True else 204
+        if updated is False:
+            return '', 204
+
         result = schema.dump(obj_).data
         if result.get('links', {}).get('self') is not None:
             result['links']['self'] = request.path
         self.after_post(result)
-        return result, status_code
+        return result, 200
 
     @check_method_requirements
     def patch(self, *args, **kwargs):
         """Update a relationship
         """
-        self.before_patch(args, kwargs)
-
         json_data = request.get_json()
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
@@ -405,6 +396,8 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
                 if obj['type'] != related_type_:
                     raise InvalidType('/data/type', 'The type provided does not match the resource type')
 
+        self.before_patch(args, kwargs, json_data=json_data)
+
         obj_, updated = self._data_layer.update_relationship(json_data,
                                                              model_relationship_field,
                                                              related_id_field,
@@ -416,19 +409,19 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
             includes.append(relationship_field)
         schema = compute_schema(self.schema, dict(), qs, includes)
 
-        status_code = 200 if updated is True else 204
+        if updated is False:
+            return '', 204
+
         result = schema.dump(obj_).data
         if result.get('links', {}).get('self') is not None:
             result['links']['self'] = request.path
         self.after_patch(result)
-        return result, status_code
+        return result, 200
 
     @check_method_requirements
     def delete(self, *args, **kwargs):
         """Delete relationship(s)
         """
-        self.before_delete(args, kwargs)
-
         json_data = request.get_json()
 
         relationship_field, model_relationship_field, related_type_, related_id_field = self._get_relationship_data()
@@ -451,6 +444,8 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
                 if obj['type'] != related_type_:
                     raise InvalidType('/data/type', 'The type provided does not match the resource type')
 
+        self.before_delete(args, kwargs, json_data=json_data)
+
         obj_, updated = self._data_layer.delete_relationship(json_data,
                                                              model_relationship_field,
                                                              related_id_field,
@@ -489,19 +484,19 @@ class ResourceRelationship(with_metaclass(ResourceMeta, Resource)):
     def after_get(self, result):
         pass
 
-    def before_post(self, args, kwargs):
+    def before_post(self, args, kwargs, json_data=None):
         pass
 
     def after_post(self, result):
         pass
 
-    def before_patch(self, args, kwargs):
+    def before_patch(self, args, kwargs, json_data=None):
         pass
 
     def after_patch(self, result):
         pass
 
-    def before_delete(self, args, kwargs):
+    def before_delete(self, args, kwargs, json_data=None):
         pass
 
     def after_delete(self, result):
